//
//  BarcodePlugin.cpp
//  DevApplication
//
//  Created by Andreas Schacherbauer on 15/05/15.
//  Copyright (c) 2015 Wikitude. All rights reserved.
//

#include "BarcodePlugin.h"

#include <iostream>
#include <sstream>

#import "WTBarcodePluginViewController.h"


BarcodePlugin::BarcodePlugin(int cameraFrameWidth, int cameraFrameHeight, WTBarcodePluginViewController *presentingViewController) :
Plugin("com.wikitude.ios.barcodePlugin"),
_currentlyRecognizedBarcodes(0),
_image(cameraFrameWidth, cameraFrameHeight, "Y800", nullptr, 0),
_presentingViewController(presentingViewController)
{
}

BarcodePlugin::~BarcodePlugin() {
    /* Intentionally Left Blank */
}


void BarcodePlugin::initialize() {    
    _imageScanner.set_config(zbar::ZBAR_NONE, zbar::ZBAR_CFG_ENABLE, 1);
}

void BarcodePlugin::destroy() {
    _image.set_data(nullptr, 0);
}

void BarcodePlugin::cameraFrameAvailable(const wikitude::sdk::Frame& cameraFrame_) {
    int frameWidth = cameraFrame_.getSize().width;
    int frameHeight = cameraFrame_.getSize().height;
    
    _image.set_data(cameraFrame_.getData(), frameWidth * frameHeight);
    
    int numberOfRecognizedCodes = _imageScanner.scan(_image);
    if ( numberOfRecognizedCodes != _currentlyRecognizedBarcodes ) {
        if ( numberOfRecognizedCodes ) {
            zbar::Image::SymbolIterator symbol = _image.symbol_begin();
            [_presentingViewController presentBarCodeResult:[NSString stringWithUTF8String:std::string(symbol->get_data()).c_str()]];
        } else {            
            [_presentingViewController presentBarCodeResult:nil];
        }
    }
    _currentlyRecognizedBarcodes = numberOfRecognizedCodes;
}

void BarcodePlugin::update(const std::list<wikitude::sdk::RecognizedTarget>& recognizedTargets_) {
    /* Intentionally Left Blank */
}
