//
//  SimpleYUVInputPlugin.cpp
//  DevApplication
//
//  Created by Andreas Schacherbauer on 10/03/16.
//  Copyright © 2016 Wikitude. All rights reserved.
//

#include "SimpleYUVInputPlugin.hpp"

#include "WTSimpleDeviceCamera.h"
#include "YUVFrameShaderSourceObject.hpp"

#include <fstream>
#include <iostream>
#include <vector>


SimpleYUVInputPlugin::SimpleYUVInputPlugin(__weak WTSimpleDeviceCamera* camera_)
:
InputPlugin("plugin.input.simple_yuv"),
_frameId(0),
_camera(camera_),
_initialized(false),
_running(false)
{ /* Intentionally Left Blank */ }

SimpleYUVInputPlugin::~SimpleYUVInputPlugin()
{ /* Intentionally Left Blank */ }

void SimpleYUVInputPlugin::initialize() {
    _initialized = [_camera initialize];

    if ( _initialized )
    {
        this->getFrameSettings().setInputFrameColorSpace(wikitude::sdk::FrameColorSpace::YUV_420_NV21);
        this->getFrameSettings().setInputFrameSize({640, 480});
        this->getFrameSettings().setFrameFieldOfView( [_camera fieldOfView] );
    }
}

void SimpleYUVInputPlugin::pause() {
    [_camera stopRunning];
    _running = false;
}

void SimpleYUVInputPlugin::resume(unsigned int pausedTime_) {
    if ( _initialized ) {
        _running = [_camera startRunning];
    }
}

void SimpleYUVInputPlugin::destroy() {
    [_camera shutdown];
}

// from InputPlugin
bool SimpleYUVInputPlugin::requestsInputFrameRendering() {
    return true;
}

bool SimpleYUVInputPlugin::requestsInputFrameProcessing() {
    return true;
}

bool SimpleYUVInputPlugin::allowsUsageByOtherPlugins() {
    return true;
}

void SimpleYUVInputPlugin::notifyNewImageBufferData(std::shared_ptr<unsigned char> imageBufferData) {
    notifyNewInputFrame(++_frameId, imageBufferData);
}
