//
//  WTYUVInputCamera.m
//  DevApplication
//
//  Created by Andreas Schacherbauer on 14/03/16.
//  Copyright © 2016 Wikitude. All rights reserved.
//

#import "WTSimpleYUVInputCamera.h"

#import "WTSimpleDeviceCamera.h"
#include "SimpleYUVInputPlugin.hpp"


@interface WTSimpleYUVInputCamera () <AVCaptureVideoDataOutputSampleBufferDelegate>

@property (nonatomic, strong) WTSimpleDeviceCamera                      *camera;

@property (nonatomic, assign) std::shared_ptr<SimpleYUVInputPlugin>     cameraInputPlugin;

@end

@implementation WTSimpleYUVInputCamera

- (instancetype)init
{
    self = [super init];
    if (self)
    {
        _camera = [[WTSimpleDeviceCamera alloc] initWithDevicePosition:AVCaptureDevicePositionBack preset:AVCaptureSessionPreset640x480 settings:[WTSimpleDeviceCamera defaultYUVDeviceCameraSettings] sampleBufferDelegate:self];

        _cameraInputPlugin = std::make_shared<SimpleYUVInputPlugin>(_camera);
    }

    return self;
}

- (std::shared_ptr<SimpleYUVInputPlugin>&)cameraInputPlugin
{
    return _cameraInputPlugin;
}

#pragma mark - Delegation
#pragma mark AVCaptureVideoDataOutputSampleBufferDelegate
- (void)captureOutput:(AVCaptureOutput *)captureOutput didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer fromConnection:(AVCaptureConnection *)connection
{
    CVImageBufferRef imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer);
    if ( imageBuffer )
    {
        if ( kCVReturnSuccess == CVPixelBufferLockBaseAddress(imageBuffer, kCVPixelBufferLock_ReadOnly) )
        {
            size_t frameDataSize = CVPixelBufferGetDataSize(imageBuffer);
            std::shared_ptr<unsigned char> frameData = std::shared_ptr<unsigned char>(new unsigned char[frameDataSize], std::default_delete<unsigned char[]>());
            std::memcpy(frameData.get(), CVPixelBufferGetBaseAddressOfPlane(imageBuffer, 0), frameDataSize);            
            _cameraInputPlugin->notifyNewImageBufferData(frameData);
            CVPixelBufferUnlockBaseAddress(imageBuffer, kCVPixelBufferLock_ReadOnly);
        }
    }
}

- (void)captureOutput:(AVCaptureOutput *)captureOutput didDropSampleBuffer:(CMSampleBufferRef)sampleBuffer fromConnection:(AVCaptureConnection *)connection
{

}

@end
