//
//  ViewController.m
//  Native Examples
//
//  Created by Andreas Schacherbauer on 18/04/15.
//  Copyright (c) 2015 Wikitude. All rights reserved.
//

#import "ExampleSelectionViewController.h"

#import <WikitudeNativeSDK/WTAuthorizationRequestManager.h>


#define WTExampleDefinitionKey_RequiredFeatures @"RequiredFeatures"

#define WTExampleFeatureKey_ImageTracking @"ImageTracking"
#define WTExampleFeatureKey_InstantTracking @"InstantTracking"
#define WTExampleFeatureKey_PhotoLibraryScreenshotImport @"PhotoLibraryScreenshotImport"


@interface ExampleSelectionViewController ()

@property (nonatomic, strong) NSDictionary                              *exampleFeatureDefinitions;
@property (nonatomic, strong) WTAuthorizationRequestManager             *authorizationRequestManager;

@end

@implementation ExampleSelectionViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.

    if ( !_authorizationRequestManager )
    {
        self.authorizationRequestManager = [[WTAuthorizationRequestManager alloc] init];
    }

    if ( !_exampleFeatureDefinitions )
    {
        NSURL *exampleDefinitionsURL = [[NSBundle  mainBundle] URLForResource:@"Examples" withExtension:@"plist"];
        self.exampleFeatureDefinitions = [NSDictionary dictionaryWithContentsOfURL:exampleDefinitionsURL];
    }
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (BOOL)shouldPerformSegueWithIdentifier:(NSString *)identifier sender:(id)sender
{
    /* If we’re about to show a example, check for authorization first */
    if ( [self exampleDefinitionForIdentifier:[ExampleSelectionViewController exampleIdentifierFromSegueIdentifier:identifier]] )
    {
        if ( !_authorizationRequestManager.isRequestingRestrictedAppleiOSSDKAPIAuthorization )
        {
            WTFeatures requiredFeatures = [self requiredWikitudeSDKFeaturesForExample:[ExampleSelectionViewController exampleIdentifierFromSegueIdentifier:identifier]];
            NSOrderedSet<NSNumber *> *restrictedAppleiOSSDKAPIs = [WTAuthorizationRequestManager restrictedAppleiOSSDKAPIAuthorizationsForRequiredFeatures:requiredFeatures];
            [_authorizationRequestManager requestRestrictedAppleiOSSDKAPIAuthorization:restrictedAppleiOSSDKAPIs completion:^(BOOL success, NSError * _Nonnull error) {
                if ( success )
                {
                    [self performSegueWithIdentifier:identifier sender:sender];
                }
                else
                {
                    /* Collect information to show an alert and log a message to the console that explains what is missing and how it can be fixed */
                    NSDictionary *unauthorizedAPIInfo = [[error userInfo] objectForKey:kWTUnauthorizedAppleiOSSDKAPIsKey];

                    NSMutableString *detailedAuthorizationErrorLogMessage = [[NSMutableString alloc] initWithFormat:@"The following authorization states do not meet the requirements:"];
                    NSMutableString *missingAuthorizations = [[NSMutableString alloc] initWithFormat:@"In order to use the Wikitude SDK, please grant access to the following:"];
                    for (NSString *unauthorizedAPIKey in [unauthorizedAPIInfo allKeys])
                    {
                        [missingAuthorizations appendFormat:@"\n* %@", [WTAuthorizationRequestManager humanReadableDescriptionForUnauthorizedAppleiOSSDKAPI:unauthorizedAPIKey]];

                        [detailedAuthorizationErrorLogMessage appendFormat:@"\n%@ = %@", unauthorizedAPIKey, [WTAuthorizationRequestManager stringFromAuthorizationStatus:[[unauthorizedAPIInfo objectForKey:unauthorizedAPIKey] integerValue] forUnauthorizedAppleiOSSDKAPI:unauthorizedAPIKey]];
                    }

                    NSLog(@"%@", detailedAuthorizationErrorLogMessage);

                    UIAlertController *settingsAlertController = [UIAlertController alertControllerWithTitle:@"Required API authorizations missing" message:missingAuthorizations preferredStyle:UIAlertControllerStyleAlert];
                    [settingsAlertController addAction:[UIAlertAction actionWithTitle:@"Open Settings" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
                        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:UIApplicationOpenSettingsURLString]];
                    }]];
                    [settingsAlertController addAction:[UIAlertAction actionWithTitle:@"NO" style:UIAlertActionStyleDestructive handler:^(UIAlertAction * _Nonnull action) {}]];

                    [self presentViewController:settingsAlertController animated:YES completion:nil];
                }

                [self.tableView deselectRowAtIndexPath:[self.tableView indexPathForSelectedRow] animated:YES];
            }];
        }

        return NO;
    }
    /* If there is no example definition, it’s a application internal segue which can continue */
    else
    {
        return YES;
    }
}

- (IBAction)trackingMapRecordingDone:(UIStoryboardSegue *)segue {    
}

- (IBAction)finishedShowingSDKInformation:(UIStoryboardSegue *)segue {
    NSOperatingSystemVersion ios8_4_1 = (NSOperatingSystemVersion){8, 4, 1};
    if ( ![[NSProcessInfo processInfo] isOperatingSystemAtLeastVersion:ios8_4_1] )
    {
        [self dismissViewControllerAnimated:YES completion:nil];
    }
}

#pragma mark - Private Methods
+ (NSString *)exampleIdentifierFromSegueIdentifier:(NSString *)segueIdentifier
{
    NSString *exampleIdentifier = nil;
    if ( segueIdentifier )
    {
        exampleIdentifier = [segueIdentifier stringByReplacingOccurrencesOfString:@"Segue" withString:@""];
    }

    return exampleIdentifier;
}

- (WTFeatures)requiredWikitudeSDKFeaturesForExample:(NSString *)exampleIdentifier
{
    WTFeatures requiredFeatures = 0;

    NSDictionary *example = [self exampleDefinitionForIdentifier:exampleIdentifier];

    id requiredFeaturesArray = [example objectForKey:WTExampleDefinitionKey_RequiredFeatures];
    if ( requiredFeaturesArray )
    {
        if ( [requiredFeaturesArray isKindOfClass:[NSArray class]] )
        {
            for (NSString *featureString in requiredFeaturesArray)
            {
                if ( [featureString isEqualToString:WTExampleFeatureKey_ImageTracking] )
                {
                    requiredFeatures |= WTFeature_ImageTracking;
                }
                else if ( [featureString isEqualToString:WTExampleFeatureKey_InstantTracking] )
                {
                    requiredFeatures |= WTFeature_InstantTracking;
                }
                else if ( [featureString isEqualToString:WTExampleFeatureKey_PhotoLibraryScreenshotImport] )
                {
                    requiredFeatures |= WTFeature_PhotoLibraryScreenshotImport;
                }
                else
                {
                    NSLog(@"Unknown or unsupported feature definition found!");
                }
            }
        }
        else
        {
            NSLog(@"Required features are not an array. Unable to parse non array feature definitions.");
        }
    }
    else
    {
        NSLog(@"Unable to access required features for example '%@'. Example definition: %@", exampleIdentifier, example);
    }

    return requiredFeatures;
}

- (NSDictionary *)exampleDefinitionForIdentifier:(NSString *)exampleIdentifier
{
    NSDictionary *exampleDefinition = nil;

    id exampleDefinitionDictionary = [_exampleFeatureDefinitions objectForKey:exampleIdentifier];
    if ( exampleDefinitionDictionary )
    {
        if ( [exampleDefinitionDictionary isKindOfClass:[NSDictionary class]] )
        {
            exampleDefinition = exampleDefinitionDictionary;
        }
    }

    return exampleDefinition;
}

@end
