//
//  WikitudeSDKOptionsDataSource.m
//  DevApplication_ExternalRendering
//
//  Created by Andreas Schacherbauer on 22/06/15.
//  Copyright (c) 2015 Wikitude. All rights reserved.
//

#import "WTCameraControlsModel.h"

#import <WikitudeNativeSDK/WikitudeNativeSDK.h>


@interface WTCameraControlsModel ()
@property (nonatomic, strong) NSMutableArray            *optionOptions;

@property (nonatomic, strong) NSArray                   *cameraPositionOptions;
@property (nonatomic, strong) NSArray                   *cameraFocusModes;
@property (nonatomic, strong) NSArray                   *continuousAutoFocusModes;

@property (nonatomic, assign) NSUInteger                numberOfFocusDistances;
@property (nonatomic, strong) NSNumberFormatter         *focusDistanceFormatter;
@property (nonatomic, strong) NSArray                   *focusDistances;

@property (nonatomic, strong) NSArray                   *torchModes;

@end

@implementation WTCameraControlsModel

+ (AVCaptureDevicePosition)devicePositionForRow:(NSInteger)row
{
    AVCaptureDevicePosition devicePosition = (AVCaptureDevicePosition)row;
    return devicePosition;
}

+ (AVCaptureFocusMode)focusModeForRow:(NSInteger)row
{
    AVCaptureFocusMode focusMode = (AVCaptureFocusMode)row;
    return focusMode;
}

+ (AVCaptureAutoFocusRangeRestriction)autoFocusRangeRestrictionForRow:(NSInteger)row
{
    AVCaptureAutoFocusRangeRestriction autoFocusRangeRestriction = (AVCaptureAutoFocusRangeRestriction)row;
    return autoFocusRangeRestriction;
}

+ (AVCaptureTorchMode)torchModeForRow:(NSInteger)row
{
    AVCaptureTorchMode torchMode = (AVCaptureTorchMode)row;
    return torchMode;
}

- (instancetype)init
{
    self = [super init];
    if (self) {
        _optionOptions = [@[] mutableCopy];
        _cameraPositionOptions = @[@"Unspecified", @"Back", @"Front"];
        _cameraFocusModes = @[@"Locked", @"Once", @"Continuous"];
        _continuousAutoFocusModes = @[@"None", @"Near", @"Far"];


        _numberOfFocusDistances = 21;
        _focusDistanceFormatter = [[NSNumberFormatter alloc] init];
        [_focusDistanceFormatter setNumberStyle:NSNumberFormatterDecimalStyle];
        [_focusDistanceFormatter setMaximumFractionDigits:2];

        NSMutableArray<NSString *> *temporaryFocusDistances = [[NSMutableArray alloc] initWithCapacity:_numberOfFocusDistances];
        CGFloat focusDistance = 0.0;
        for (NSUInteger i = 0; i <= _numberOfFocusDistances; i++)
        {
            [temporaryFocusDistances addObject:[_focusDistanceFormatter stringFromNumber:@(focusDistance)]];
            focusDistance += 0.05;
        }
        _focusDistances = [[NSArray alloc] initWithArray:temporaryFocusDistances];

        _torchModes = @[@"Off", @"On", @"Auto"];
    }

    return self;
}

#pragma mark - Public Methods
- (void)updatePickerView:(UIPickerView *)pickerView
{
    NSInteger currentlySelectedMainRow = [pickerView selectedRowInComponent:0];
    BOOL shouldAnimateChanges = YES;
    
    /* Check if capture device positions needs an update */
    if ( 0 == currentlySelectedMainRow ) {
        AVCaptureDevicePosition currentlyActiveCaptureDevicePosition = [self.delegate currentlyActiveCameraPositionForCameraControlsModel:self];
        NSInteger activeCaptureDevicePositionRow = (NSInteger)currentlyActiveCaptureDevicePosition;
        
        [pickerView selectRow:activeCaptureDevicePositionRow inComponent:1 animated:shouldAnimateChanges];
    }
    /* or focus mode */
    else if ( 1 == currentlySelectedMainRow )
    {
        AVCaptureFocusMode currentlyActiveCaptureFocusMode = [self.delegate currentlyActiveFocusModeForCameraControlsModel:self];
        NSInteger activeCaptureDeviceFocusModeRow = (NSInteger)currentlyActiveCaptureFocusMode;

        [pickerView selectRow:activeCaptureDeviceFocusModeRow inComponent:1 animated:shouldAnimateChanges];
    }
    /* or auto focus range restriction */
    else if ( 2 == currentlySelectedMainRow )
    {
        AVCaptureAutoFocusRangeRestriction currentlyActiveAutoFocusRangeRestriction = [self.delegate currentlyActiveAutoFocusRangeRestrictionForCameraControlsModel:self];
        NSInteger activeAutoFocusRangeRestrictionRow = (NSInteger)currentlyActiveAutoFocusRangeRestriction;

        [pickerView selectRow:activeAutoFocusRangeRestrictionRow inComponent:1 animated:shouldAnimateChanges];
    }
    /* or focus distance */
    else if ( 3 == currentlySelectedMainRow )
    {
        float currentlyActiveFocusDistance = [self.delegate currentlyActiveCameraFocusDistanceForCameraControlsModel:self];
        NSInteger activeFocusDistanceRow = (float)currentlyActiveFocusDistance / 0.05;

        [pickerView selectRow:activeFocusDistanceRow inComponent:1 animated:shouldAnimateChanges];
    }
    /* or torch mode */
    else if ( 4 == currentlySelectedMainRow )
    {
        AVCaptureTorchMode currentlyActiveTorchMode = [self.delegate currentlyActiveTorchModeForCameraControlsModel:self];
        NSInteger activeTorchModeRow = (NSInteger)currentlyActiveTorchMode;

        [pickerView selectRow:activeTorchModeRow inComponent:1 animated:shouldAnimateChanges];
    }
}

#pragma mark - Delegation
#pragma mark UIPickerViewDataSource
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    return 2;
}
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    NSInteger numberOfRowsInComponent = 0;

    if ( 0 == component )
    {
        _optionOptions = [self.delegate validateAvailableCameraOptions:[@[@"Position", @"Focus Mode", @"Auto Focus Restriction", @"Focus Distance", @"Torch Mode"] mutableCopy]];
        numberOfRowsInComponent = [_optionOptions count];
    }
    else if ( 1 == component )
    {
        if ( 0 == [pickerView selectedRowInComponent:0] ) {
            numberOfRowsInComponent = [_cameraPositionOptions count];
        }
        else if ( 1 == [pickerView selectedRowInComponent:0] )
        {
            numberOfRowsInComponent = [_cameraFocusModes count];
        }
        else if ( 2 == [pickerView selectedRowInComponent:0] )
        {
            numberOfRowsInComponent = [_continuousAutoFocusModes count];
        }
        else if ( 3 == [pickerView selectedRowInComponent:0] )
        {
            numberOfRowsInComponent = _numberOfFocusDistances;
        }
        else if ( 4 == [pickerView selectedRowInComponent:0] )
        {
            numberOfRowsInComponent = [_torchModes count];
        }
    }

    return numberOfRowsInComponent;
}

- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
    NSString *titleForRow = @"";

    if ( 0 == component )
    {
        titleForRow = [_optionOptions objectAtIndex:row];
    }
    else if ( 1 == component )
    {
        if ( 0 == [pickerView selectedRowInComponent:0] ) {
            titleForRow = [_cameraPositionOptions objectAtIndex:row];
        }
        else if ( 1 == [pickerView selectedRowInComponent:0] )
        {
            titleForRow = [_cameraFocusModes objectAtIndex:row];
        }
        else if ( 2 == [pickerView selectedRowInComponent:0] )
        {
            titleForRow = [_continuousAutoFocusModes objectAtIndex:row];
        }
        else if ( 3 == [pickerView selectedRowInComponent:0] )
        {
            titleForRow = [_focusDistances objectAtIndex:row];
        }
        else if ( 4 == [pickerView selectedRowInComponent:0] )
        {
            titleForRow = [_torchModes objectAtIndex:row];
        }
    }

    return titleForRow;
}

#pragma mark UIPickerViewDelegate
- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component
{
    if ( 0 == component ) {
        [pickerView reloadComponent:component +1];
        [self updatePickerView:pickerView];
    }
    if ( 1 == component ) {
        if ( 0 == [pickerView selectedRowInComponent:0] )
        {
            [self.delegate cameraControlsModel:self didSelectCameraPosition:[WTCameraControlsModel devicePositionForRow:row]];
        }
        else if ( 1 == [pickerView selectedRowInComponent:0] )
        {
            [self.delegate cameraControlsModel:self didSelectCameraFocusMode:[WTCameraControlsModel focusModeForRow:row]];
        }
        else if ( 2 == [pickerView selectedRowInComponent:0] )
        {
            [self.delegate cameraControlsModel:self didSelectContinuousAutoFocusMode:[WTCameraControlsModel autoFocusRangeRestrictionForRow:row]];
        }
        else if ( 3 == [pickerView selectedRowInComponent:0] )
        {
            NSString *focusDistanceString = [_focusDistances objectAtIndex:row];
            float focusDistance = [[_focusDistanceFormatter numberFromString:focusDistanceString] floatValue];            
            [self.delegate cameraControlsModel:self didSelectCameraFocusDistance:focusDistance];
        }
        else if ( 4 == [pickerView selectedRowInComponent:0] )
        {
            [self.delegate cameraControlsModel:self didSelectTorchMode:[WTCameraControlsModel torchModeForRow:row]];
        }
    }
}


@end
